<script type="text/javascript">
  function showSuggestions(editor, suggestions) {
    var cur = editor.getCursor(false), token = editor.getTokenAt(cur);
    // Build the select widget
    var complete = document.createElement("div");
    complete.className = "completions";
    var sel = complete.appendChild(document.createElement("select"));
    sel.multiple = true;
    for (var i = 0; i < suggestions.length; ++i) {
      var text = suggestions[i].text;
      var opt = sel.appendChild(document.createElement("option"));
      opt.value = text;
      if (suggestions[i].category) text = suggestions[i].category + ' - ' + text;
      opt.appendChild(document.createTextNode(text));
    }
    sel.firstChild.selected = true;
    sel.size = Math.min(10, suggestions.length);
    var pos = editor.cursorCoords();
    complete.style.left = pos.x + "px";
    complete.style.top = pos.yBot + "px";
    document.body.appendChild(complete);
    // Hack to hide the scrollbar.
    if (suggestions.length <= 10)
      complete.style.width = (sel.clientWidth - 1) + "px";
    var done = false;
    function close() {
      if (done) return;
      done = true;
      complete.parentNode.removeChild(complete);
    }
    function pick() {
      editor.replaceRange(sel.options[sel.selectedIndex].value, 
        {line: cur.line, ch: token.start}, {line: cur.line, ch: token.end});
      close();
      setTimeout(function(){editor.focus();}, 50);
    }
    function again() {
      startAutocomplete(editor);
    }
    $(sel).blur(close);
    $(sel).dblclick(pick);
    $(sel).keydown(function(event) {
      var code = event.keyCode;
      // Enter and space
      if (code == 13 || code == 32) {event.stopPropagation(); pick();}
      // Escape
      else if (code == 27) {event.stopPropagation(); close(); editor.focus();}
      else if (code != 38 && code != 40) {close(); editor.focus(); setTimeout(again, 50);}
    });
    sel.focus();
    // Opera sometimes ignores focusing a freshly created node
    if (window.opera) setTimeout(function(){if (!done) sel.focus();}, 100);
    return true;
  }
</script>
<?php
  function theme_highlighter_load_ac($files) {
    $first = true;
    foreach ($files as $filename => $type) {
      $category = '';
      $f = fopen('../plugins/theme_highlighter/'.$filename, "r");
      while (($line = fgets($f)) !== false) {
        $line = trim($line);
        if (substr($line,0,2) == '//') {
          $category = trim(substr($line,2));
        } else if ($line != '' && (!$category || $category == 'GetSimple' || ($pos = strpos($line,'(')) === false || function_exists(substr($line,0,$pos)))) {
          if (!$first) echo ",\r\n        "; else $first = false;
          echo '{ mode:'.json_encode($type).', text:'.json_encode($line).', category:'.json_encode($category).' }';
        }
      }
      fclose($f);
    }
  }
  function theme_highlighter_load_hl($files) {
    $first = true;
    foreach ($files as $filename => $style) {
      $category = '';
      $f = fopen('../plugins/theme_highlighter/'.$filename, "r");
      while (($line = fgets($f)) !== false) {
        $line = trim($line);
        if ($line != '' && substr($line,0,2) != '//' && preg_match('/^([$A-Za-z0-9_]+)/', $line, $matches)) {
          if (!$first) echo ",\r\n        "; else $first = false;
          echo json_encode($matches[1]) . ':' . json_encode($style);
        }
      }
      fclose($f);
    }
  }

  if (basename($_SERVER['PHP_SELF']) == 'theme-edit.php') {
    $selector = '#codetext';
    $filename = @$_GET['f'];
    if (strrpos($filename,'.css') === strlen($filename)-4) {
      $mode = 'css';
    } else if (strrpos($filename,'.js') === strlen($filename)-3) {
      $mode = 'javascript';
    } else {
      $mode = 'php';
    }
  } else if (basename($_SERVER['PHP_SELF']) == 'components.php') {
    $selector = 'textarea[name^=val]';
    $mode = 'php';
  }
  if ($mode == 'php') {
?>
    <script type="text/javascript">
      var ac = [
        <?php theme_highlighter_load_ac(array('ac_php_getsimple.txt'=>'php','ac_php_plugins.txt'=>'php','ac_php.txt'=>'php','ac_css.txt'=>'css')); ?> 
      ];
      var hl = {
        <?php theme_highlighter_load_hl(array('ac_php.txt'=>'function','ac_php_getsimple.txt'=>'function-2','ac_php_plugins.txt'=>'function-3')); ?> 
      };
      function startAutocomplete(editor) {
        // We want a single cursor position.
        if (editor.getSelection()) return;
        // Find the token at the cursor
        var cur = editor.getCursor(false), token = editor.getTokenAt(cur), s = token.string.trim().toLowerCase();
        var mode = token.state.stack[token.state.stack.length-1].name;
        var suggestions = [];
        for (var i=0; i<ac.length; i++) {
          if (ac[i].mode == mode && (s == null || s == '' || ac[i].text.substring(0,s.length).toLowerCase() == s)) {
            suggestions.push(ac[i]);
          }
        }
        if (suggestions.length > 0) showSuggestions(editor,suggestions);
      }
  
      $(function() {
        $(<?php echo json_encode($selector); ?>).each(function(i,textarea) {
          var editor = CodeMirror.fromTextArea(textarea, {
            lineNumbers: true,
            matchBrackets: true,
            mode: { name:<?php echo json_encode($mode); ?>, styles:hl },
            indentUnit: 2,
            indentWithTabs: true,
            enterMode: "keep",
            tabMode: "shift",
            functions: "substr strlen",
            onKeyEvent: function(editor, e) {
              // Hook into ctrl-space
              if (e.keyCode == 32 && (e.ctrlKey || e.metaKey) && !e.altKey) {
                e.stop();
                return startAutocomplete(editor);
              }
            }
          });
        });
      });
    </script>
<?php
  } else {
?>
    <script type="text/javascript">
      var ac = {
        <?php theme_highlighter_load_ac(array('ac_css.txt'=>'')); ?> 
      }
      $(function() {
        $(<?php echo json_encode($selector); ?>).each(function(i,textarea) {
          var editor = CodeMirror.fromTextArea(textarea, {
            lineNumbers: true,
            matchBrackets: true,
            mode: <?php echo json_encode($mode); ?>,
            indentUnit: 2,
            indentWithTabs: true,
            enterMode: "keep",
            tabMode: "shift"
          });
        });
      });
    </script>
<?php
  }


